"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tab = exports.Tabs = exports.createMdxProvider = void 0;
const style_module_css_1 = __importDefault(require("./style.module.css"));
const react_1 = require("@mdx-js/react");
const react_enterprise_alert_1 = __importDefault(require("@hashicorp/react-enterprise-alert"));
const react_tabs_1 = __importDefault(require("@hashicorp/react-tabs"));
// This function returns a simple object containing the default components
// The `additionalComponents` param is purely for convenience.
// It is intended for use with `next-mdx-remote`.
function defaultMdxComponents({ product, additionalComponents = {}, }) {
    return Object.assign(_defaultComponents(product), additionalComponents);
}
exports.default = defaultMdxComponents;
// This function returns an MDXProvider that can be used when rendering
// within an mdx layout.
// It is intended for use with `next-mdx-enhanced`. Once we no longer use
// `next-mdx-enhanced` on any of our properties, it will be depreated.
function createMdxProvider({ product, additionalComponents = {} }) {
    const allComponents = Object.assign(_defaultComponents(product), additionalComponents);
    return function CustomMDXProvider({ children }) {
        return React.createElement(react_1.MDXProvider, { components: allComponents }, children);
    };
}
exports.createMdxProvider = createMdxProvider;
// Purely for sharing between the two functions. Once `createMdxProvider` is
// deprecated, this can be moved inline.
function _defaultComponents(product) {
    return { Tabs, Tab, EnterpriseAlert: createEnterpriseAlert(product) };
}
//
// Base components need to be slightly modified to fit this context
//
// Enterprise alert gets the "product" prop defaulted
function createEnterpriseAlert(product) {
    return function EnterpriseAlert(props) {
        return React.createElement(react_enterprise_alert_1.default, Object.assign({ product: product }, props));
    };
}
// Tabs is a general-purpose component that we format for ease of use within mdx
// It is also wrapped in a span with a css module class for styling overrides
function Tabs({ children }) {
    if (!Array.isArray(children))
        throw new Error('Multiple <Tab> elements required');
    return (React.createElement("span", { className: style_module_css_1.default.tabsRoot },
        React.createElement(react_tabs_1.default, { items: children.map((Block) => ({
                heading: Block.props.heading,
                // eslint-disable-next-line react/display-name
                tabChildren: () => Block,
            })) })));
}
exports.Tabs = Tabs;
// This is a simple helper that makes it more clear for docs authors
function Tab({ children }) {
    return React.createElement(React.Fragment, null, children);
}
exports.Tab = Tab;
